﻿using Infragistics.Win;
using Infragistics.Win.UltraWinEditors;
using Infragistics.Win.UltraWinGrid;
using System;
using System.Collections.Generic;
using System.ComponentModel;
using System.Data;
using System.Drawing;
using System.Globalization;
using System.Linq;
using System.Text;
using System.Threading.Tasks;
using System.Windows.Forms;

namespace UGMultiCellPaste
{
    public partial class Form1 : Form
    {
        public Form1()
        {
            InitializeComponent();
        }

        private void ultraGrid1_BeforeMultiCellOperation(object sender, Infragistics.Win.UltraWinGrid.BeforeMultiCellOperationEventArgs e)
        {
            bool flag = false;
            UltraGrid grid = (UltraGrid)sender;
            if (e.Operation == MultiCellOperation.Paste)
            {
                foreach (UltraGridCell cell in e.Cells)
                {
                    if (cell.Value.ToString() != "")
                    {
                        var result = MessageBox.Show("Some of the cells to paste within \n" +
                             "have existing data. Do you want to overwrite the existing values?", "Warning", MessageBoxButtons.OKCancel);

                        if (result == DialogResult.Cancel)
                        {
                            e.Cancel = true;
                            return;
                        }
                        else
                        {
                            break;
                        }
                    }
                }

                foreach (UltraGridCell cell in e.Cells)
                {
                    if (cell.Column.Key.ToString().Equals("Column 0"))
                    {
                        bool passedValidation = this.HandleIntValues(cell, e.NewValues[cell].Value.ToString());
                        if (!passedValidation)
                        {
                            e.Cancel = true;
                            break;
                        }
                    }
                    if (cell.Column.Key.ToString().Equals("DropDown Col"))
                    {
                        bool passedValidation = this.HandleDropDownValues(cell, e.NewValues[cell].Value.ToString());
                        if (!passedValidation)
                        {
                            e.Cancel = true;
                            break;
                        }
                    }
                    if (cell.Column.Key.ToString().Equals("DateColumn"))
                    {
                        bool passedValidation = this.HandleDateTimeValues(cell, e.NewValues[cell].Value.ToString());
                        if (!passedValidation)
                        {
                            e.Cancel = true;
                            break;
                        }
                        else
                        {
                            CultureInfo enUS = new CultureInfo("en-US");
                            DateTime dt = DateTime.ParseExact(e.NewValues[cell].Value.ToString(), "dd-MM-yyyy", enUS);
                            cell.Value = dt;
                            flag = true;
                        }
                    }
                    if (flag)
                    {
                        flag = false;
                        e.Cancel = true;
                    }
                }
            }
        }

        private void Form1_Load(object sender, EventArgs e)
        {
            UltraComboEditor combo = new UltraComboEditor();
            List<string> vals = new List<string>() { "val 1", "val 2", "val 3" };

            foreach (string val in vals)
            {
                combo.ValueList.ValueListItems.Add(val);
            }
            combo.DropDownStyle = DropDownStyle.DropDownList;
            this.ultraGrid1.DisplayLayout.Bands[0].Columns["DropDown Col"].EditorComponent = combo;
        }

        private void ultraGrid1_Error(object sender, ErrorEventArgs e)
        {
            Console.WriteLine(e.ErrorType);
            if (e.MultiCellOperationErrorInfo != null && e.ErrorType == ErrorType.MultiCellOperation)
            {
                InvalidOperationException exception = e.MultiCellOperationErrorInfo.Exception as InvalidOperationException;
                if (exception != null)
                {
                    string exceptionStartRows = "Contents being pasted have more rows than what's available starting from the anchor cell. Paste contents have ";
                    string exceptionStartCols = "Contents being pasted have more columns than what's available starting from the anchor cell. Paste contents have ";
                    string exceptionMessage = exception.Message;
                    if (exceptionMessage.StartsWith(exceptionStartRows))
                    {
                        MessageBox.Show("The pasted vlaues exceed the available rows count!");

                        // Parse the exception message to determine the number of rows that we would need in
                        // or to successfully complete the Paste operation. 
                        exceptionMessage = exceptionMessage.Substring(exceptionMessage.Length - 2, 1);

                        int rowCountAvailable = int.Parse(exceptionMessage);
                        Console.WriteLine(rowCountAvailable);

                        // The ancor cell should be the grid's active cell
                        UltraGridCell activeCell = this.ultraGrid1.ActiveCell;
                        this.HandleExceedPastedContent(activeCell);
                        
                        e.Cancel = true;
                    } else if (exceptionMessage.StartsWith(exceptionStartCols))
                    {
                        MessageBox.Show("The pasted vlaues exceed the available columns count!");
                        // Parse the exception message to determine the number of columns that we would need in
                        // or to successfully complete the Paste operation. 
                        exceptionMessage = exceptionMessage.Substring(exceptionMessage.Length - 2, 1);

                        int colCountAvailable = int.Parse(exceptionMessage);
                        Console.WriteLine(colCountAvailable);

                        UltraGridCell activeCell = this.ultraGrid1.ActiveCell;
                        this.HandleExceedPastedContent(activeCell);

                        e.Cancel = true;
                    }
                }
            } 
        }

        private void HandleExceedPastedContent(UltraGridCell activeCell)
        {
            if (activeCell != null)
            {
                string[] clpText = Clipboard.GetText().Split('\n');
                int cellRowIndex = activeCell.Row.Index;
                int cellColumnIndex = activeCell.Column.Index;
                for (int i = 0; i < this.ultraGrid1.Rows.Count - cellRowIndex; i++)
                {
                    string[] curRowVals = clpText[0 + i].Split('\t');
                    for (int j = 0; j < curRowVals.Count(); j++)
                    {
                        if (cellColumnIndex + j < this.ultraGrid1.Rows[cellRowIndex].Cells.Count)
                        {
                            string curVal = curRowVals[j].Trim();
                            UltraGridCell curCell = this.ultraGrid1.Rows[cellRowIndex + i].Cells[cellColumnIndex + j];

                            if (curCell.Column.Key.ToString().Equals("Column 0"))
                            {
                                bool passedValidation = this.HandleIntValues(curCell, curVal);
                                if (passedValidation)
                                {
                                    Type cellType = curCell.Column.DataType;
                                    curCell.Value = Convert.ChangeType(curVal, cellType);
                                }
                            }
                            else if (curCell.Column.Key.ToString().Equals("DropDown Col"))
                            {
                                bool passedValidation = this.HandleDropDownValues(curCell, curVal);
                                if (passedValidation)
                                {
                                    Type cellType = curCell.Column.DataType;
                                    curCell.Value = Convert.ChangeType(curVal, cellType);
                                }
                            }
                            else if (curCell.Column.Key.ToString().Equals("DateColumn"))
                            {
                                bool passedValidation = this.HandleDateTimeValues(curCell, curVal);
                                if (passedValidation)
                                {
                                    CultureInfo enUS = new CultureInfo("en-US");
                                    DateTime dt = DateTime.ParseExact(curVal, "dd-MM-yyyy", enUS);
                                    curCell.Value = dt;
                                }
                            }
                            else
                            {
                                Type cellType = curCell.Column.DataType;
                                curCell.Value = Convert.ChangeType(curVal, cellType);
                            }
                        }
                    }
                }
            }
        }

        private bool HandleDateTimeValues(UltraGridCell cell, string newValue)
        {
            CultureInfo enUS = new CultureInfo("en-US");
            DateTime dDate;
            bool isOfRequiredFormat = DateTime.TryParseExact(newValue, cell.Column.Format, enUS, DateTimeStyles.None, out dDate);
            if (!isOfRequiredFormat)
            {
                MessageBox.Show("Pasted date values should match the exact date format!");
                return false;
            }
            else
            {
                return true;
            }
        }

        private bool HandleIntValues(UltraGridCell cell, string newValue)
        {
            if (Int32.Parse(newValue) < 0 || Int32.Parse(newValue) > 10)
            {
                MessageBox.Show("All integers must be greater than 0 and less than 10");
                return false;
            } else
            {
                return true;
            }
        }

        private bool HandleDropDownValues(UltraGridCell cell, string newValue)
        {
            if (!((cell.Column.EditorComponent as UltraComboEditor).ValueList.FindByDataValue(newValue) != null))
            {
                MessageBox.Show("Pasted values for \"DropDown Col\" should be of the possible items!");
                return false;
            }
            else
            {
                return true;
            }

        }
    }
}
